package java.lang.invoke;

import java.util.Objects;
import java.lang.reflect.Modifier;
import java.lang.reflect.Member;

public interface MethodHandleInfo
{
    public static final int REF_getField = 1;
    public static final int REF_getStatic = 2;
    public static final int REF_putField = 3;
    public static final int REF_putStatic = 4;
    public static final int REF_invokeVirtual = 5;
    public static final int REF_invokeStatic = 6;
    public static final int REF_invokeSpecial = 7;
    public static final int REF_newInvokeSpecial = 8;
    public static final int REF_invokeInterface = 9;
    
    public abstract int getReferenceKind();
    
    public abstract Class<?> getDeclaringClass();
    
    public abstract String getName();
    
    public abstract MethodType getMethodType();
    
    public abstract <T extends Member> T reflectAs(Class<T> p0, MethodHandles.Lookup p1);
    
    public abstract int getModifiers();
    
    default public boolean isVarArgs() {
        return !MethodHandleNatives.refKindIsField((byte)this.getReferenceKind()) && Modifier.isTransient(this.getModifiers());
    }
    
    public static String referenceKindToString(int i) {
        if (!MethodHandleNatives.refKindIsValid(i)) {
            throw MethodHandleStatics.newIllegalArgumentException("invalid reference kind", Integer.valueOf(i));
        }
        return MethodHandleNatives.refKindName((byte)i);
    }
    
    public static String toString(int n, Class<?> clazz, String obj, MethodType obj2) {
        Objects.requireNonNull(obj);
        Objects.requireNonNull(obj2);
        return String.format("%s %s.%s:%s", new Object[] { referenceKindToString(n), clazz.getName(), obj, obj2 });
    }
}
